;======================== header ===============================================
;-
; NAME:
;   cactus_c3_clean
; PURPOSE:
;   does the tricky non-default cleaning (eg removal of planets) specific for cactus.
;   This routine removes bright like point sources such as cosmic ray hits, planets and 
;   stars from LASCO images.
;   This routine is optimised for removing as much as possible.
; CATEGORY:
;   CACTUS
; INPUTS:
;    
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
; PROCEDURE:
;
; CALLS:
;   
; MODIFICATION HISTORY:
;   2002/03/28, ver 1.00, David Berghmans
;               created on the basis of db_cosmics
;   2002/05/30, DB, output pixel values cannot be bigger than input
;   2003/03/12, Kristel Jeanquart, removal of 'mask=bytarr' because there were two of it
;   2003/12/13, DB, test 'if ncosmics GT 0' added  
;   2005/01/06, ER, better correction for rotation of SOHO; now image itself is corrected in clean.pro     
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added 
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/13, ER, incorporate cactus_clean into cactus_c2_clean and cactus_c3_clean
;   2008/03/14, ER, change the rotation s.t. it is done similarly as in secchi_prep, update also header info (index.crota)
;   2008/03/17, ER, Move the info on corrected cosmics to index.history[15]
;   2008/04/02, ER, sccwritefits the image and index after being cleaned
;   2008/05/13, JMK, make detector dir always lowercase [strlowcase]
;   2008/05/13, JMK ncosmics parameter added
;   2008/06/09, ER, do not save fits files anymore locally
;   2009/02/15, ER, Use keyword spatial_clean instead of no_clean (more logical)
;
;-
;======================== end header ===========================================
PRO cactus_c3_clean, filename=filename, image,index,ncosmics, no_logfile=no_logfile,spatial_clean=spatial_clean

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

 ; --------------------------- MAIN ------------------------------------

   image = LASCO_READFITS(filename, index, /silent)

   returned_value=GET_EXP_FACTOR(index,efac,bias)
                ; 0 for success, 
                ; -1 for file not found,
                ; -2 for time not found.
   index.history[9]=returned_value
   IF returned_value NE 0 THEN efac = 1.
   	   	; (the efac values are only known for older data, 
   		;  eg on April 26, 2002, efac was available up till Jan 2002 on 
      		;  /net/cronus/data/opt/local/ssw/soho/lasco/idl/expfac/data only!)
   mask= readfits(dir.var+'c3clearmask.fts')
   image = (float(image) - bias)*mask ; we use floats to avoid extra discretisation noise in further steps
	
 ; normalize to exposure per sec and correct for rotation of SOHO
   image = image/(index.exptime*efac)
   image = rot(image,-1*index.crota1, 1,index.crpix1, index.crpix2,/interp,/pivot,missing=0)  ; rotates image counter-clockwise (-1), around occulter center (crpix1,2)

 ; update header information
   index.crota=DOUBLE(0)  

 ; Clean the image
 ; Makes a median image, and thresholds on pixels exceeding the median
 ; image by N*sqrt(median). A different N is used close to the central occulter.

    ncosmics=0

IF KEYWORD_SET(spatial_clean) THEN BEGIN

 ; initialisations 
   xcen   =  index.crpix1  
   ycen   =  index.crpix2 
   dilate_str=replicate(1,3,3)

   in_image=image
   
   r=200.
   mask=BYTARR(1024,1024)
   for x=xcen-r,xcen+r do mask[x, (-sqrt(r^2-(x-xcen)^2)+ycen):(sqrt(r^2-(x-xcen)^2)+ycen)]=1

   boxsize = 11
   med_im  = median(in_image,boxsize)    
   diff    = in_image - med_im
   sigma= sqrt(med_im)

   hits    = (diff GT 0.5*sigma and not mask)
   hits=dilate(hits, dilate_str)             ; important !!!
   hitsloc = where( hits , ncosmics)
   if ncosmics GT 0 THEN BEGIN
      image(hitsloc) = med_im(hitsloc)
      med_im  =median(image,boxsize)  ; recalculate the median image to minimise influence of stars and cosmics
      image(hitsloc) = med_im(hitsloc)
   endif else printf, log, 'Anomaly: no cosmics found!!!. Ncosmics=', ncosmics  
  
   occulter=BYTARR(1024,1024)
   r=80.
   for x=xcen-r,xcen+r do occulter[x, (-sqrt(r^2-(x-xcen)^2)+ycen):(sqrt(r^2-(x-xcen)^2)+ycen)]=1
   boxsize = 9
   med_im  = median(in_image,boxsize)    
   diff    = in_image - med_im
   sigma= sqrt(med_im)

   hits    =  (diff GT 1*sigma and mask and not occulter)
   hits=dilate(hits, dilate_str)             ; important !!!
   hitsloc = where( hits , ncosmics)
   if ncosmics GT 0 THEN BEGIN
      image(hitsloc) = med_im(hitsloc)
      med_im  =median(image,boxsize)  ; recalculate the median image to minimise influence of stars and cosmics
      image(hitsloc) = med_im(hitsloc)
   endif else print, log, 'Anomaly: no cosmics found!!!. Ncosmics=', ncosmics  
  

   printf, log, ' ncosmics corrected=',ncosmics
   index.history[15]=' CACTus ncosmics corrected= '+strtrim(ncosmics,2)   

   image = image < in_image  ; output pixel values cannot be bigger than input

 ENDIF ELSE printf, log, 'Skipped spatial cleaning'
;----------------------------closing----------------------------------------------------------
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END

