;======================== header ===============================================
;+
; NAME: cactus_cme_txt
;    
; PURPOSE:
;   This routine calculates in which images the movie is visible, based on the speed of the CME. 
;   It copies the correct images into the CME directories
;
; CALLS:
;    cactus_cme_ID.pro
;    circle.pro 
;    strnr.pro
; OUTPUTS:
;    CME.txt
;    
;
; MODIFICATION HISTORY:
;   2006/03/01, ER, created, version bin-2.1.0
;   ---------------------- version 2.1.0 --------------------------------
;   2007/08/08, ER&SH, call the script makeMov to make a quicktime movie per CME
;   ---------------------- version 2.2.0 --------------------------------
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/15, ER, removed npixelx and npixely as parameters, they not used at the moment
;   2008/01/16, ER, now call cacus_cme_txt directly from out.pro
;   2008/01/17, ER, makemov is now an IDL function
;   2008/01/30, ER, replace spawns with file_copy and file_delete
;   2008/03/12, ER, Remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/14, ER, extract 'absolute' cactus parameters from !cactusfov and not from fov.sav
;   2008/03/24, ER, replace telescope with detector and use cactus_detectors to find the detector
;   2008/03/25, ER, adapt radius r0 for plotting inner circle on COR2 images
;   2008/04/03, ER, add keyword no_movie
;   2008/05/23, ER, add keyword tempdir, usd dir.tmp when calling makemov inside cactus
;   2009/05/22, ER, change keyword no_movie into movie: default is no_movie
;   2009/10/30, ER, correct deletion of image files after making CME.mov for reducing memory useage
;-    
;======================== end header ===========================================
PRO cactus_cme_txt, no_logfile=no_logfile, movie=movie, instrument=instrument,cmecat

  cactus_preamble, dir,myname
  cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------

 detector=(cactus_detectors(instrument))[0] 
 imlabel=instrument+'-'+detector
 
 restore, dir.var+'julstart.sav'		;julstart

 restore, dir.work+detector+'index.sav'   ; index
 
 ; determine size of images
 caldat, double(julstart.int+julstart.frac)+index[0].obstime, mm,dd,yyyy,ho,mi,se    
    yy=STRMID(STRTRIM(yyyy,2),2,2) & mm=STRNR(mm,2) & dd=STRNR(dd,2) 
    ho=STRNR(ho,2) & mi=STRNR(mi,2) & se=STRNR(se,2) 
    im=imlabel+'_'+yy+mm+dd+'.'+ho+mi+se+'.jpeg'
    read_jpeg,dir.images+im,im0
 
 sz=size(im0)
 npixelx=sz[1]
 npixely=sz[2]
 printf, log, ' The images have size: ',strtrim(npixelx,2),' x ',strtrim(npixely,2)

; determine graphics device
   ;device, decomposed=0
      printf, log, ' Working in batchmode, using Z-buffer'
      set_plot, 'Z'
      device, set_resolution = [npixelx,npixely]   ; size of image
      device, z_buffering    = 0           ; 0 for 2D

 ; prepare for screen output
   ;device, decomposed=0
   tvlct, myr,myg,myb,/get
   RED  = [255,255 ,    0,    0,   0,  255]	       ;  red=1,  green=2
   GREEN= [0  ,255,   255,  255,   0,    0]	       ; blue=3, yellow=4, purple=5
   BLUE = [0  ,  0,     0,  255,  255, 255]
   myr[0:5]=RED
   myg[0:5]=GREEN
   myb[0:5]=BLUE
   tvlct, myr,myg,myb
 

 

 for i=1,n_elements(cmecat)-1 do begin
   thiscme=cmecat[i]
   cactus_cme_ID, thiscme, ID, yyyy,mm,dd,ho,mi,se,thisdir		;cactus function which extracts the ID, year, month, day, etc ...
   cmedir=dir.out+thisdir
 
  ; initialize
    CASE detector OF 
	'c2' :  r0=37 
	'cor2': r0=25
   ENDCASE
   r1=180
   xc=128 & yc=128
   c1=circle(xc,yc,r0) & 	c2=circle(xc,yc,r1) 	;routine

;################################
;#	 MAKE CME.TXT           #
;################################					

   printf, log, STRMID(i,4,4),'. ',STRMID(thiscme.type,0,3),' ',STRNR(thiscme.nr),' ----> ',thiscme.date,' ',strmid(thiscme.time,0,8)

   npixelr=float(index[0].maxradius-index[0].minradius) ;(48-15=33 pixels of the cactusmap are for c2)   
    							;(224-28=196 pixels of the cactusmap are for c3)

   fov_in_km 		=  npixelr*!cactusfov.km_per_pix	
   fov_crossing_time	= (fov_in_km/thiscme.speed)/(60.*60.*24.)		;(these are days)
   
   first=min(abs(thiscme.jultime - index.obstime),first_im)  ; first_im is the index of the images closest to the start-time of the CME
   first_im=max([first_im-3,1])
   diff=min(abs((thiscme.jultime+(thiscme.dt0)/24.+fov_crossing_time)-index.obstime),last_im)	
   middle_im=first_im+(last_im-first_im)/3
	
   openw,cme_txt, cmedir+'CME.txt', /get_lun  	; iu stands for indexunit
   n_im=0
   FOR j=first_im,last_im DO BEGIN
    caldat, double(julstart.int+julstart.frac)+index[j].obstime, mm,dd,yyyy,ho,mi,se    
    yy=STRMID(STRTRIM(yyyy,2),2,2) & mm=STRNR(mm,2) & dd=STRNR(dd,2) 
    ho=STRNR(ho,2) & mi=STRNR(mi,2) & se=STRNR(se,2) 
    im=imlabel+'_'+yy+mm+dd+'.'+ho+mi+se+'.jpeg'
    dateline=index[j].date_obs+' '+STRMID(index[j].time_obs,0,8)
    printf, cme_txt, im,' "',dateline,'"'
    if j eq first_im then printf, log, '      * first im - ',im
    if j eq last_im  then printf, log, '         last im - ',im
    n_im=n_im+1
   read_jpeg,dir.images+im,im0
    tv, im0

;################################
;#	 CME INFO ON IMAGE      #
;################################					

    pa=thiscme.angle & da=thiscme.width	;pada are degrees (scaled to 360)
    p1=(pa-da/2.+360)MOD 360 & p2=(pa+da/2.)MOD 360 ;edges, measured from solar north
    t1=p1*!pi/180. & t2=p2*!pi/180. 
    plots, [c1[0,p1],c2[0,p1]],[c1[1,p1],c2[1,p1]],/device,thick=1
    plots, [c1[0,p2],c2[0,p2]],[c1[1,p2],c2[1,p2]],/device,thick=1
    IF p1 lt p2 THEN plots,  c1[0,p1:p2], c1[1,p1:p2],/device,thick=1 $
    ELSE  BEGIN  plots,  c1[0,p1:359], c1[1,p1:359],/device,thick=1 
                 plots,  c1[0,0:p2], c1[1,0:p2],/device,thick=1
    ENDELSE 
    xyouts, 110,128,'CACTus',/device, charsize=0.8
  ; save the image
    write_jpeg, cmedir+im,tvrd(),quality=100
      
   ENDFOR
   printf, log, '            # im = ',strtrim(n_im,2)
   close, cme_txt
   free_lun, cme_txt  
   

  ; copy middle image
    caldat, double(julstart.int+julstart.frac)+index[middle_im].obstime, mm,dd,yyyy,ho,mi,se    
    yy=STRMID(STRTRIM(yyyy,2),2,2) & mm=STRNR(mm,2) & dd=STRNR(dd,2) 
    ho=STRNR(ho,2) & mi=STRNR(mi,2) & se=STRNR(se,2) 
    im=imlabel+'_'+yy+mm+dd+'.'+ho+mi+se+'.jpeg'
    printf, log, '      * CME.jpeg - ',im  
    file_copy, cmedir+im, cmedir+'CME.jpeg',/overwrite
   
   ; make CME.mov
   ; from Sheng

IF KEYWORD_SET(movie) THEN BEGIN
   printf, log, '      * CME.mov - '
   printf, log, ' Call makemov.pro'
   moviename=cmedir+'CME.mov'
   success=makemov(cmedir,infiles=imlabel+'*.jpeg',tempdir=dir.tmp+thisdir,outfile=moviename,ffmpegdir=dir.bin,ffmpegoptions="-r 6") 
   IF success THEN BEGIN 
	printf, log, ' Movie was made successful: ',moviename 
	files=file_search(cmedir,imlabel+'_*.jpeg',count=count)
   	file_delete,files, /allow_nonexistent
   ENDIF ELSE printf, log, ' ATTENTION! Problem with making movie: ',moviename
ENDIF
endfor

; ------------------------closing remarks-------------------------------
   cactus_loginfo, myname, log, no_logfile=no_logfile, /close
  
END
