;======================== header ===============================================
;-
; NAME:
;   cactus_COR2_clean 
; PURPOSE: 
;   does the tricky non-default cleaning (eg removal of planets) specific for cactus.
;   This routine removes cosmic bright point like sources such as ray hits, planets and 
;   stars from LASCO images.
;   This routine is optimised for removing as much as possible.
; CATEGORY:
;   CACTUS
; INPUTS:
;    
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
; PROCEDURE:
;
; CALLS:
;   
; MODIFICATION HISTORY:
;
;   2008/03/13, ER, Created based on cor2_clean_cactus made in Sept 2007
;   2008/03/14, ER, turn on the updating of the header, e.g. index.crota changes after rotate_on
;   2008/03/17, ER, Move the info on corrected cosmics to index.history[15]
;   2008/04/02, ER, sccwitefits the cleaned images in e.g. dir.in/COR2/ to speed up program
;   2008/04/03, ER, revert the above, this routine is now called from cactus_preprocess and .xdr.gz images are written there
;   2008/05/12, ER, index.crpix1 and 2 are originally doubles, make them double again to avoid rounding off errors in calculation
;   2008/05/14, ER, take cactus_preamble out and also no_logfile, it is now cactus independent
;   2008/05/14, ER, put additional mask for cor2-B images-testing
;   2008/05/15, ER, made the mask smaller so that cosmics close to the occulter are also removed
;   2008/05/15, ER, put the time in index.time_obs
;   2008/06/17, JMK, added no_clean keyword
;   2009/02/15, ER, Pass keyword spatial_clean instead of no_clean (more logical)
;   2009/07/06, ER, put keyword /interp in secchi_prep
;   2009/09/01, ER, Insert a routine to find and remove planets (in images where a planet was found)
;-
;======================== end header ===========================================
PRO cactus_COR2_clean, filename=filename,image,index,ncosmics, spatial_clean=spatial_clean

; ------------------------main code ------------------------------------

 secchi_prep, filename, index, image, /silent $
        ,outsize=1024           $       ;resizes the image and updates header (xcen,naxis...)
        ,fill_value=0           $       ;sets all missing pixels to NaN, find back with z=finite(image,/NaN)
        ,/calfac_off            $       ;switches off conversion to msb (mean solar brightness), i.e. keep it in DN
        ,/calimg_off            $       ;if set it does not correct for vignetting
        ,/rotate_on     	$       ;rotates the image to solar noth up, with the occulter center (crpix1,2) as rotation axis
	,/interp			;to avoid bad images when rotate_on is set, if not set nearest neighbor is used.

;; Alternative method to avoid secchi_prep
;image=sccreadfits(filename, index)*1.
;mask=get_smask(index)
;image=(image-index.biasmean)/index.exptime
;s=2
;image=rebin(image*mask,2048/s,2048/s)
;index.crpix1=index.crpix1/s & index.crpix2=index.crpix2/s
;index.naxis1=index.naxis1/s & index.naxis2=index.naxis2/s
;index.cdelt1=index.cdelt1*s	;platescale

index.time_obs=strmid(index.date_obs,11,12)   ; index.time_obs is originally empty in secchi headers
 
; ---------------------------removal of cosmics, stars and planets------------------------------------
 ;   Makes a median image, and thresholds on pixels exceeding the median
 ;  image by N*sqrt(median). A different N is used close to the central occulter.

 ncosmics=0
 
 image=remove_planet(image,index,target='venus')
 image=remove_planet(image,index,target='mercury')

 IF KEYWORD_SET(spatial_clean) THEN BEGIN
 
 ; initialisations (remark: index.crota1 is either 0 or 180 and indicates whether SOHO is rotated or not
   xcen   =   double(index.crpix1) ; sun center position in x-dir
   ycen   =   double(index.crpix2) ; sun center position in y-dir
   dilate_str=replicate(1,3,3)
     
   in_image=image
   mask=BYTARR(index.naxis1,index.naxis2)
   r=double(200.*index.naxis1/2048.)
   for x=xcen-r,xcen+r do mask[x, (-sqrt(r^2-(x-xcen)^2)+ycen):(sqrt(r^2-(x-xcen)^2)+ycen)]=1
   boxsize = 11
   
   med_im  = median(in_image,boxsize)
   diff    = in_image - med_im
   sigma   = sqrt(med_im)   ; this supposes that the DN follows a poisson distribution
   hits    = (diff GT 0.8*sigma and  not mask)
   hits    = dilate(hits, dilate_str)             ; important !!! 
   hitsloc = where( hits , ncosmics)

   if ncosmics GT 0 THEN BEGIN
      image(hitsloc) = med_im(hitsloc)
      med_im  = median(image,boxsize)  ; recalculate the median image to minimise influence of stars and cosmics
      image(hitsloc) = med_im(hitsloc)
   endif else print, 'Anomaly: no cosmics found!!!. Ncosmics=', ncosmics
 
   index.history[15]=' CACTus ncosmics corrected = '+ strtrim(ncosmics,2)   

 ENDIF

 scc_mask=get_smask(index)
 image=image*scc_mask
END
