;+
; NAME: 
;   cactus_create_regions
;
; PURPOSE:
;   This function extracts the different connected regions in a cube with 
;   difference images.
;
; USE:
;     regions = cactus_create_regions(diffcube
;                                     [,high_thresh=high_thresh]
;                                     [,low_thresh=low_thresh]
;                                     [,mask1=mask1]
;                                     [,mask2=mask2]
;                                     [,opening_mask1=opening_mask1]
;                                  )
;
; INPUTS:
;     parameters:
;         diffcube:   the cube in wich the regions are
;                     searched. 
;
;     Keywords:
;         high_thresh:    the highest threshold. This threshold will
;                         be used to select the mayor clear points.
;
;         low_thresh:     the lowest threshold. This threshold
;                         minimal intensity of the point wich will 
;                         be added to a region
;
; OUTPUTS:
;     Keywords:
;         mask1:          when keyword is set this keyword will
;                         contain the mask1
;
;         mask2:          when keyword is set this keyword will 
;                         contain the mask2
;
;         opening_mask1:  when keyword is set this keyword will
;                         contain the opening of mask1
;
;     Returns:
;         A cube with the same dimensions of the given diffcube. The
;         elements in this have values starting from zero wich is the
;         background and for every region a different number. Points
;         who are part of the same region have the same value.
;
;     MODIFICATION HISTORY:
;        2010/08/12, BB, Created
; 
;======================================================================
;-
FUNCTION cactus_create_regions, $
   diffcube, $
   high_thresh=high_thresh, $
   low_thresh=low_thresh, $
   mask1=mask1, $
   mask2=mask2, $
   opening_mask1=opening_mask1


  nimages  = (size(diffcube))[3]
  nrad = (size(diffcube))[2]
  nang = (size(diffcube))[1] 

  high_t = 80
  low_t = 40

  IF KEYWORD_SET(high_thresh) THEN high_t = high_thresh
  IF KEYWORD_SET(low_thresh) THEN low_t = low_thresh


  strucElem8 = intarr(3,3)
  strucElem8[[1,3,4,5,7]]=1
  
  scaled_cube = diffcube GT high_t
  IF KEYWORD_SET(mask1) THEN mask1 = scaled_cube

  regions = INTARR(nang,nrad,nimages)
  FOR imind=0, nimages-1 DO BEGIN
     image = scaled_cube[*,*,imind] *0
     loc_image = diffcube[*,*,imind]
     peak_loc = where(scaled_cube[*,*,imind] gt 0, npeak_loc)
     IF npeak_loc GT 0 THEN BEGIN
        locgrow = region_grow(loc_image, $
                          peak_loc,$
                          threshold=[low_thresh,max(loc_image)],$
                          /all_neighbors)
        if n_elements(locgrow) GT 1 then image[locgrow] = 1
     ENDIF
     region = label_region(image) 
     hist = histogram(region)
     to_small = where(hist lt 5, nto_small)
     FOR i_ts = 0, nto_small-1 DO BEGIN
        loc_to_small = where(region EQ to_small[i_ts],n_loc)
        IF n_loc GT 0 THEN image[loc_to_small]=0
     ENDFOR
     image = label_region(image)
     regions[*,*,imind]=image
  ENDFOR
  scaled_cube=0 
  RETURN, regions  
END
