;======================== header ===============================================
;+
; NAME:
;  CACTUS_INCUBE
; PURPOSE:
;   This is a preprocessing module that selects the good images, cleans them 
;   and finally reformats the input images.
;
; CATEGORY:
;   CACTUS 
;
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /no_logfile   : to print diagnostics to a no_logfile (default=stdout=screen)
;   detector   : 'LASCO' or 'SECCHI'
; OUTPUTS:
;
; COMMON BLOCKS: 
;  INDEX 
;
; CALLS:
;   cactus_loginfo
;   cactus_c2_clean
;   cactus_c3_clean
;
; MODIFICATION HISTORY:
;   2002/03/28, ver 1.00, David Berghmans
;               created on the basis of spwp/db_c3_select, db_c3_cleaner, db_c3_reformat
;   2002/05/23, DB, incube can handle the situation now when an image is read it in, that 
;                   was processed before.
;   2003/02/14, KJ, incube now checks if same image has arrived via 2 different fits files,
;                   if so the last one is removed
;   2003/04/24, DB, cactusmap adaptions (no more c2_reformat, c3_reformat)
;   2003/05/04, KJ, doesn't call c2_clean_cactus or c3_clean_cactus anymore
;   2003/06/10, DB, pipe dir removed
;   2005/06/27, DB, no more saving to archive, fits files are now just deleted from in-dir  
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/28, ER, make telescope a keyword, should be set to 'c2' or 'c3'
;   2008/01/28, ER, fix bug in badimages part, if mybadpos eq nmyfiles-1 then do not shift images forward
;   2008/03/12, ER, remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/13, ER, replace telescope with instrument
;   2008/03/13, ER, replace findfile with file_search, findfile is an obsolete routine since IDL6.1
;   2008/03/13, ER, only perform cactus_image_select for lasco images
;   2008/03/13, ER, incorporate cactus_clean into cactus_c2_clean and cactus_c3_clean
;   2008/03/14, ER, extract npixelx, npixely, npixela and npixelr from !cactusfov and not from fov.sav
;   2008/03/17, ER, rename cactus_COR2_clean.pro with cactus_cor2_clean.pro
;   2008/03/17, ER, Replace 'instrument' with 'detector' this is the name of 'c2' and 'c3' and 'COR2' in the header
;   2008/03/18, ER, Retrieve detectors (c2/c3/COR2) form function cactus_detectors and give instrument keyword instead of detector as input
;   2008/03/18, ER, correct error: replace detector with detectors[d]   
;   2008/04/02, ER, do not download the original fits-files, images are read in directly in cactus_*_clean.pro
;   2008/05/15, JMK Added skip if no files where found
;   2008/05/15, JMK updated calls (none to cactus_image_select)
;   2008/06/12, JMK removed conversion from flt to int for xycube
;   2008/06/12, JMK, Added time filtering correctly
;   2008/08/29, JMK, time filtering optionally with no_time_clean [deafult on]
;   2009/02/15, ER, take out saving of *_b4clean_*_images.sav, they are not used, it saves 0.5 G disk space
;   2009/10/29, ER, insert in commented style the lines needed for the
;                   Bram_cleaning, thresholds need to be checked, does not remove stars
;   2010/08/12, BB, Added cactus_morf_time_cleaning
;-
;======================== end header ===========================================
PRO cactus_incube, no_logfile=no_logfile , instrument=instrument, no_time_clean=no_time_clean

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------
 ; settings: size of output arrays

 ; loop over detectors 
   detectors=cactus_detectors(instrument)

   for d=0,n_elements(detectors)-1 do begin

  	 myfiles=file_search(dir.in+detectors[d]+path_sep()+'*_cactus.xdr.gz', count=nmyfiles)

         if nmyfiles eq 0 then GOTO,SKIPDETECTOR

         new_xy_images=FLTARR(!cactusfov.npixelx,!cactusfov.npixely, nmyfiles) 
         new_ar_images=FLTARR(!cactusfov.nangles,!cactusfov.nradii, nmyfiles) 
 	 FOR i=0, nmyfiles-1 DO BEGIN
             s=readstruct(myfiles[i]) 
	     new_xy_images[*,*,i]= s.im 
             new_ar_images[*,*,i]  = s.polarim
             IF (i eq 0) THEN myindex=s.hdr ELSE myindex=[myindex, s.hdr]
       ENDFOR      



        ; check if same image has arrived via 2 different fits files
	  badimages=BYTARR(nmyfiles)
	    ;##########
	    printf, log, 'nmyfiles=',nmyfiles
	     ;##########
	 FOR image_i=0, nmyfiles-2 DO BEGIN
	    FOR image_j=image_i+1, nmyfiles-1 DO BEGIN
	          seenbefore= (myindex[image_j].date_obs EQ myindex[image_i].date_obs AND $
	                       myindex[image_j].time_obs EQ myindex[image_i].time_obs )
                   
	          IF seenbefore THEN BEGIN badimages[image_j]=1
	     ;##########
		       
                     printf, log, myindex[image_i].filename
                     printf, log, myindex[image_i].date_obs
                     printf, log, myindex[image_i].time_obs
                     printf, log, myindex[image_i].obstime
                     printf, log, myindex[image_j].filename
                     printf, log, myindex[image_j].date_obs
                     printf, log, myindex[image_j].time_obs
                     printf, log, myindex[image_j].obstime
	     ;##########
		  ENDIF
           ENDFOR
        ENDFOR      


	  badpos=where(badimages, nbad)
		    ;#########
	  printf, log, 'badpos=',badpos
	  printf, log, 'nbad=',nbad
		    ;#########
	  
          IF nbad gt 0 THEN BEGIN          
	     ; write log info
	     printf, log, 'There have been ', nbad,' images that arrived via different fits files:'
	  
	     FOR i=0, nbad-1 DO BEGIN
	       printf, log, myindex[badpos[i]].date_obs, '  ', myindex[badpos[i]].time_obs
	     ENDFOR
	  
	     mybadpos=where(badimages, stillbad) 
	     WHILE stillbad GT 0  AND NOT (mybadpos[0] eq nmyfiles-1) DO BEGIN
		printf, log, 'Removing double images...'
		mybadpos=mybadpos[0]
		printf, log, '   stillbad=',stillbad, '  mybadpos=', mybadpos
	        myindex[mybadpos:(nmyfiles-2)]=myindex[(mybadpos+1):*]
	  	new_xy_images[*,*,mybadpos:(nmyfiles-2)]=new_xy_images[*,*,(mybadpos+1):*]
	        new_ar_images[*,*,mybadpos:(nmyfiles-2)]=new_ar_images[*,*,(mybadpos+1):*]
		
;                badimages[mybadpos]=0
                badimages[mybadpos:(nmyfiles-2)]=badimages[(mybadpos+1):*]
		badimages[nmyfiles-1]=0  
	        mybadpos=where(badimages, stillbad) 
	     ENDWHILE
	  	
	     ; remove bad end 
             ngood=nmyfiles-nbad
	     new_xy_images=new_xy_images[*,*,0:(ngood-1)]
	     new_ar_images=new_ar_images[*,*,0:(ngood-1)]
	     myindex= myindex[0:(ngood-1)]
	     nmyfiles=ngood
	     
	     printf, log, '...done'
	  ENDIF   



        ; Writing new data to incube: check if sav files exist, if not initialise
        ; If they do exist, we must carefully check if each new image is perhaps already present
        ; in the sav files.
          fname_ar_images=dir.work+detectors[d]+'_ar_images.sav'
          fname_xy_images=dir.work+detectors[d]+'_xy_images.sav'
          fname_index    =dir.work+detectors[d]+'index.sav'
          all_files_exist = (file_exist(fname_ar_images) and $
	                     file_exist(fname_xy_images) and $
	                     file_exist(fname_index)     )
          ;all_files_exist = 0
          IF all_files_exist THEN BEGIN 
	         printf, log, 'Output files exist, appending...'
               restore, fname_ar_images     ; contains polar
               restore, fname_xy_images     ; contains xycube
               restore, fname_index
 	         good=replicate(1,nmyfiles)   ; all 1 by default, good[i]=1 when image i is not present in .sav file
 	         oldnimages    =(size(polar))[3]   
	         ; avoid trouble with wrong xycube size
 	         old_xy_nimages=(size(xycube))[3]  
	         IF oldnimages lt old_xy_nimages THEN BEGIN
		    	  xycube=xycube[*,*,(old_xy_nimages-oldnimages):*]
		    	  printf, log, ' XYCUBE MISMATCH ERROR: oldnimages lt old_xy_nimages'
		   ENDIF   
		 IF oldnimages gt old_xy_nimages THEN BEGIN
		    xycube=INTARR(!cactusfov.npixelx, !cactusfov.npixely, oldnimages)
                printf, log, ' XYCUBE MISMATCH ERROR: oldnimages gt old_xy_nimages'
		 ENDIF   
	       ; 
	       where_new_images = BYTARR(oldnimages + nmyfiles) ; certainly long enough, will be shortened later if necessary
               where_new_images[oldnimages:*] = 1
             FOR j=0, nmyfiles-1 DO BEGIN
	            seenbefore= where(myindex[j].date_obs EQ index.date_obs AND $
	                            myindex[j].time_obs EQ index.time_obs )
	            ; if we have seen it before, replace the old version with the new version
                  IF (max(seenbefore) ne -1) THEN BEGIN   
                     index[seenbefore]      = myindex[j]
                     polar[*,*,seenbefore]  = new_ar_images[*,*,j]
                     xycube[*,*,seenbefore] = new_xy_images[*,*,j]
                     good[j] = 0
                     where_new_images[seenbefore]     = 1
                     printf, log, myindex[j].filename+' existed before and was replaced with new version'
	            ENDIF  
	       ENDFOR
	       keepthese=where(good,nmyfiles) ; ! nmyfiles gets new value
	       where_new_images=where_new_images[0:(oldnimages -1 + nmyfiles)] ; adapt to new value of nmyfiles
	       IF nmyfiles GT 0 THEN BEGIN
                  all_ar_images=FLTARR(!cactusfov.nangles,!cactusfov.nradii,oldnimages + nmyfiles)
                  all_ar_images[*,*,0:(oldnimages-1)]=polar
                  all_ar_images[*,*,oldnimages:*]    =new_ar_images[*,*,keepthese]
                  polar=all_ar_images
			all_ar_images=0b  ; save memory

                  all_xy_images=INTARR(!cactusfov.npixelx,!cactusfov.npixely,oldnimages + nmyfiles)
                  all_xy_images[*,*,0:(oldnimages-1)]=xycube
                  all_xy_images[*,*,oldnimages:*]    =new_xy_images[*,*,keepthese]
                  xycube=all_xy_images
			all_xy_images=0b  ; save memory
                  FOR j=0, nmyfiles-1 DO index  =[index,     myindex[keepthese[j]]]
	       ENDIF   
          ENDIF ELSE BEGIN     ; the case of no pre-existing sav files
	       printf, log, 'Output files do not exist, creating...'
             where_new_images = replicate(1,nmyfiles)
             polar  = new_ar_images 
	       xycube = new_xy_images
             index  = myindex 
          ENDELSE
        ; the images in polar are not necessarily in chronological order. Therefore we have to sort:
          printf, log, 'Sort the images in the right order'  
	  good_order=sort(index.obstime)
          index=index[good_order]
          nimages=n_elements(index)
	  ;
          temp=FLTARR(!cactusfov.nangles,!cactusfov.nradii,nimages)
          for i=0,nimages-1 do temp[*,*,i]=polar[*,*,good_order[i]]
          polar=temp  
	  ;
          temp=FLTARR(!cactusfov.npixelx,!cactusfov.npixely,nimages) ; JMK INT -> FLT
          for i=0,nimages-1 do temp[*,*,i]=xycube[*,*,good_order[i]]
          xycube=temp & temp = 0b
	  
          where_new_images=where_new_images[good_order]
	    
	 ; JMK Cleaning
   	  ;if not keyword_set(no_time_clean) then begin
	  ;  polar=filter_3d_time(polar,width=3,sigma=3)
	 ;   xycube=filter_3d_time(xycube,width=3,sigma=3)   
    	  ;endif
	    
	 ; BRAM cleaning
  	if not keyword_set(no_time_clean) then begin
          xycube=filter_3d_time(xycube,width=3,sigma=3)
	  polar= cactus_morf_time_cleaning(polar, index,high_thresh=0.009,low_thresh=0.007) ; thresholds to be checked
        endif

        ; save    
          save, filename=fname_ar_images, polar
          save, filename=fname_xy_images, xycube
          save, filename=fname_index, index 
     
     
     
        ; dump all variables related to this detector
          all_ar_images   = 0b
          all_xy_images   = 0b
          polar           = 0b
	    xycube          = 0b
          new_ar_images   = 0b
          new_xy_images   = 0b
          myfiles         = 0b

  SKIPDETECTOR:	  
  endfor  ; detectors
; ------------------------closing remarks-------------------------------
 
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
