;+
; NAME:
;   CACTUS
;
; PURPOSE:
;   Prodecure to detect CMEs using the CACTus Method
;
; INPUTS:
;   startdate=20060406
;   [enddate] if enddate is not given it is set the same as startdate
;   if startdate and enddate are not given (both) then real-time application is assumed for today and 4 past days
;
; KEYWORD PARAMETERS:
;   thres  : if set it will be the threshold for detection, default is 0.3
;   width  : if set it will be the threshold for CME widths, default is 8
;   click  : if set, a clickable map will be produced in html, default is off
;   secchi : if set, it will look for secchi images (default is LASCO)
;   sc	   : values are 'a' or 'b' , default is 'a'
;   no_logfile: if set, no_logfiles will be saved in cactus/log
; OUTPUTS:
;   general output and outputs per CME
;
; MODIFICATION HISTORY:
;   2007/12/14, E. Robbrecht   Created	
;   2008/01/09, ER, maindir given as keyword 
;   2008/01/11, ER, delete fitsfiles from previous run out of in/c2 and in/c3
;   2008/01/28, ER, cactus_incube now called with keyword 'telescope'
;   2008/03/10, ER, remove cactus_settings from cactus_preamble
;   2008/03/11, ER, add instrument keywords /secchi and /qkl
;   2008/03/12, ER, maindir keyword removed from all routines except cactus_master, only needed in cactus_settings now
;   2008/03/13, ER, add secchi keyword in calling cactus_getdata
;   2008/03/13, ER, replace telescope with instrument in cactus_incube
;   2008/03/17, ER, Replace 'instrument' with 'detector' this is the name of 'c2' and 'c3' and 'COR2' in the header
;   2008/03/18, ER, add the instrument keyword to cactus_incube (instead of detector) and cactus_reldev
;   2008/03/20, ER, add maxniterations as keyword to cactus_velmap
;   2008/03/21, ER, add instrument keyword to cactus_catalo and cactus_out
;   2008/03/26, ER, change default settings: click_map: ON, silent: ON, logfile: ON
;   2008/03/27, ER, Remove the silent keyword
;   2008/04/03, ER, add no_movie keyword to out
;   2008/04/03, ER, add keywords p1 and p2 to ONLY do part1 or part2 of the master
;   2008/04/07, ER, if in real-time mode, enable keyword /qkl
;   2008/04/10, ER, sc (a or b) added as keyword
;   2008/06/28, JMK, added cactus_to_archive call (twice)
;   2008/07/22, ER, only perform cactus_to_archive if keyword is set
;   2008/07/22, JMK, added no_time_clean & no_spatial_clean keyword to make cleaning optional
;   2009/02/13, ER, add factor to the master logfile
;   2009/02/13, ER, change keyword no_spatial_clean to more logical spatial_clean, default is spatial_clean=0
;   2009/03/02, ER, add keyword factor to cactus_out, for passing it to cactus_header
;   2009/05/22, ER, change keyword no_movie into movie: default is no_movie
;   2009/05/22, ER, part 1 of program now only preprocesses the images, part 2 starts at incube
;   2010/02/03, ER, Changed the IF statements for /p1 and /p2: if /p1 and /p2 are both set, both will be performed (= default)
;   2010/02/03, ER, Changed the log-information
;-
;==========================================================================================================================
PRO cactus_master, ndays=ndays, startdate, enddate, secchi=secchi, sc=sc, qkl=qkl, $
				thresh=thresh,width=min_w, maxniterations=maxniterations, maindir=maindir,factor=factor, $
				no_logfile=no_logfile, no_click_map=no_click_map, movie=movie, $
				p1=p1, p2=p2, help=help,archive=archive,no_time_clean=no_time_clean,spatial_clean=spatial_clean

IF KEYWORD_SET(help) THEN cactus_help ELSE BEGIN

IF n_params() eq 0 THEN qkl=1

  cactus_settings, maindir=maindir, secchi=secchi, qkl=qkl
  cactus_preamble, dir,myname,hostname
  cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------ keyword settings ----------------------------
   IF NOT KEYWORD_SET(thresh) THEN IF NOT KEYWORD_SET(sc)  THEN thresh = 0.3 ELSE thresh = 0.25	; set default intensity threshold -- this is good for LASCO and SECCHI-A; for SECCHI-B use 0.25
   IF NOT KEYWORD_SET(factor) THEN IF NOT KEYWORD_SET(secchi)  THEN factor = 4 ELSE factor=6	; set default factor for threshold in velmap.pro	-- good for LASCO. FOR SECCHI use factor=6 as default
   IF NOT KEYWORD_SET(min_w) 	THEN min_w = 5    	; set default width threshold (degrees)
   IF NOT KEYWORD_SET(maxniterations) THEN maxniterations=300   ; set default number of iterations
   IF NOT KEYWORD_SET(secchi) 	THEN instrument='LASCO' ELSE instrument='SECCHI' ; set default instrument to LASCO
   IF NOT KEYWORD_SET(sc)	THEN sc='a'  
   IF NOT KEYWORD_SET(no_time_clean) THEN no_time_clean=0 ; set default to cleaning in time ; JMK
   IF NOT KEYWORD_SET(spatial_clean) THEN spatial_clean=0 ; set default to no median cleaning in space (it is slow)
   
;Default: if no keyword is given (p1 or p2) the entire program will run
IF NOT KEYWORD_SET(p1) AND NOT KEYWORD_SET(p2) THEN BEGIN
	p1=1 & p2=1
ENDIF


;<--start part1
IF KEYWORD_SET(p1) THEN BEGIN

   IF KEYWORD_SET(archive)	THEN BEGIN
      printf, log, ' '
      printf, log, '0. Move current output directory to archive'
      cactus_to_archive,archive=archive,/overwrite
   ENDIF
   
   printf, log, ' '
   printf, log, '1. Get LASCO/SECCHI data (cactus_getdata.pro)'
   IF n_params() eq 0 THEN cactus_getdata, ndays=ndays, instrument=instrument, sc=sc, no_logfile=no_logfile  $
       ELSE IF n_params() eq 1 THEN  cactus_getdata, ndays=ndays, startdate, instrument=instrument, sc=sc, no_logfile=no_logfile $
       ELSE cactus_getdata, ndays=ndays, startdate, enddate, instrument=instrument, sc=sc, no_logfile=no_logfile
   
   printf, log, ' '
   printf, log, '2. Read and preprocess individual images (cactus_preprocess.pro)'
   IF NOT KEYWORD_SET(spatial_clean) THEN printf, log, '    Images are NOT cleaned spatially (median filtering). This is the default setting.'
   cactus_preprocess, no_logfile=no_logfile,instrument=instrument,spatial_clean=spatial_clean
 ENDIF
;end part1 -->

;<-- start part2
IF KEYWORD_SET(p2) THEN BEGIN
  
   printf, log, ' '
   printf, log, '3. Read and preprocess data cube (cactus_incube.pro)'
   IF NOT KEYWORD_SET(no_time_clean) THEN printf, log, '    Image cube is cleaned in time (= default)'
   cactus_incube, no_logfile=no_logfile,instrument=instrument,no_time_clean=no_time_clean
   
   
   printf, log, ' '
   printf, log, '4. Prepare running difference cube (cactus_reldev.pro)'
   cactus_reldev, no_logfile=no_logfile, instrument=instrument

   printf, log, ' '
   printf, log, '5. Transform image into Hough space and detect ridges (cactus_velmap.pro)'
   printf, log, '    Thres: ',strtrim(thresh,2),' / factor: ',strtrim(factor,2)
   cactus_velmap, maxniterations=maxniterations,no_logfile=no_logfile, factor=factor,thresh=thresh
   
   printf, log, ' '
   printf, log, '6. Cluster points into CMEs and make collect parameters (cactus_catalo.pro)'
   printf, log, '    Thres: ',strtrim(thresh,2),' / factor: ',strtrim(factor,2),' / minimal CME width: ',strtrim(min_w,2)
   cactus_catalo, width=min_w, no_logfile=no_logfile, instrument=instrument, thresh=thresh
   
   printf, log, ' '
   printf, log, '7. Make LatestCMEs.html and all graphic material for display (cactus_out.pro)' 
   IF  KEYWORD_SET(movie) THEN printf, log, '    Movies are made'
   IF NOT KEYWORD_SET(no_click_map) THEN printf, log, '    CMEmap is clickable'
   cactus_out, no_click_map=no_click_map, movie=movie,no_logfile=no_logfile, thresh=thresh, width=min_w, instrument=instrument,factor=factor
   
   IF KEYWORD_SET(archive)	THEN BEGIN
      printf, log, ' '
      printf, log, '8. Move current output directory to archive (again)'
      cactus_to_archive,archive=archive,/overwrite
   ENDIF

ENDIF
;end part2 -->

printf, log, ' '

; ------------------------closing remarks-------------------------------
 cactus_loginfo, myname, log, no_logfile=no_logfile, /close
ENDELSE ; not in help mode


END
