;======================== header ===============================================
;+
; NAME:
;    cactus_polarmap
; PURPOSE:
;    reformats c3 and c2 images to a 'cactusmap' as specified in var.pro
; CATEGORY:
;    CACTUS
; INPUTS:
;    image, index   
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   /comment: to get input parameters for polar.so printed on the screen
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
; PROCEDURE:
;
; CALLS:
;   polar.so (in C)
;
; MODIFICATION HISTORY:
;   2002/03/28, ver 1.00, David Berghmans ('c3_reformat')
;   2003/04/24, DB, renamed 'cactusmap', works now on both c2 and c3
;                                        and make a standardized cactusmap
;   2004/02/18, ER, correction for rotation of SOHO; 
;   	    	    added a constant rotation_SOHO=1 in case of rotation (=0 if no rotation)
;   2004/02/18, ER, replace constant by index parameter: index.crot1 (=180 if rotated)
;   2004/08/31, DB, polar transformation now excuted by external call to c-routine by
;                   Bogdan Nicula
;   2005/01/06, ER, better correction for rotation of SOHO; now image itself is corrected in clean.pro     
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2007/12/14, ER, add comment keyword
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/01/30, ER&BN, introduce cactus_polar, it is a disguised c-routine which acts like an IDL FUNCTION
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/14, ER, Remove the rotation info, headers are updated in cleaning routines, crpix1,2 is the occulter center for all images of all instruments now
;   2008/03/14, ER, xcen and ycen are now always index.crpix1 and index.crpix2
;   2008/03/14, ER, define a !cmemap variable which contains the polar cmemap definitions
;   2008/03/17, ER, arcsec_per_ccd_pixel=index.cdelt1 (56.0 for c3, 11.9 for c2) and km_per_pix is a new function
;   2008/03/17, ER, add maxradius_cor2 and minradius_cor2 to fov.sav, prepare new tags for cactusindex (still commented)
;   2008/03/24, ER, adapt cactusindex (date and time) and include info on obsrvtry/instrument/detector/origin   
;   2008/04/03, ER, no logfile written
;   2008/05/15, JMK, check tag_exist before copying.
;   2008/05/16, ER, fixed the date_obs and time_obs for lasco and secchi (in cactus_cor2_clean index.time_obs is defined for secchi)
;   2008/05/16, ER, cactusindex.detector=strlowcase(index.detector) because of changes in cactus_detectors.pro
;-
;======================== end header ===========================================
FUNCTION cactus_polarmap, image, index, no_logfile=no_logfile, comment=comment 

   cactus_preamble, dir,myname

 ; ------------------------main code ------------------------------------
   restore, dir.var+'fov.sav'          
     ; minradius_c2,maxradius_c2,minradius_c3,maxradius_c3 ,minradius_cor2,maxradius_cor2
   
 ; Which detector?
   CASE strlowcase(index.detector) OF
     'c3' : BEGIN 
        maxradius = maxradius_c3
      	minradius = minradius_c3
       END
      'c2' : BEGIN
      	maxradius = maxradius_c2
      	minradius = minradius_c2
      END
      'cor2' : BEGIN
        maxradius = FIX(maxradius_cor2*(index.naxis1/2048.))
        minradius = FIX(minradius_cor2*(index.naxis1/2048.))
      END
   ENDCASE  
 ; basic solar physics stuff
   km_per_ccd_pix = km_per_pix(index)
  
 ; derived settings  
   minr_cactusmap    = ROUND(minradius*km_per_ccd_pix/!cactusfov.km_per_pix)
   ; minimal height that the current image will fill in the output cactusmap
   maxr_cactusmap    = ROUND(maxradius*km_per_ccd_pix/!cactusfov.km_per_pix)
   ; maximal height that the current image will fill in the output cactusmap
   npixel1=!cactusfov.nangles
   npixel2=maxr_cactusmap-minr_cactusmap+1
   ; number of pixels that the current image will fill in the output cactusmap  

  ; cactus_polar calls a c_routine which is compiled by make_dll in cactus_settings.pro 
   cactusmap=FLTARR(!cactusfov.nangles,!cactusfov.nradii)
   cactusmap[*,minr_cactusmap:maxr_cactusmap]=cactus_polar(image, index.crpix1, index.crpix2, minradius, maxradius, !cactusfov.edge, npixel1, npixel2)

  ; adapt/strip the index structure to something more useful
   restore, dir.var+'cactusindex.sav'
   restore, dir.var+'julstart.sav'
   cactusindex.filename       = index.filename
   cactusindex.detector       = strlowcase(index.detector)		; c2 / c3 / cor2
   cactusindex.instrume       = index.instrume 	 			; = LASCO / SECCHI
   IF tag_exist(index,'telescop') then cactusindex.obsrvtry = index.telescop   ; = SOHO
   IF tag_exist(index,'obsrvtry') then cactusindex.obsrvtry = index.obsrvtry   ; = STEREO_A / STEREO_B
   IF tag_exist(index,'origin')   then cactusindex.origin   = index.origin     ; = NRL / GSFC
   cactusindex.date_obs       = strmid(anytim2cal(index.date_obs,form=11),0,10)   ; formatting needed to streamline secchi with lasco
   cactusindex.time_obs       = index.time_obs
   cactusindex.ncosmics       = index.history[15] 
   cactusindex.expfac         = index.history[9]
   cactusindex.minradius      = minr_cactusmap
   cactusindex.maxradius      = maxr_cactusmap
   ;
   jultime            = anytim2jd(cactusindex.date_obs+' '+cactusindex.time_obs)
   cactusindex.obstime= float(jultime.int-julstart.int) +  (jultime.frac-julstart.frac)
   ;

   index=cactusindex 
  return, cactusmap  

END
