;======================== header ===============================================
;+
; NAME:
;    cactus_preprocess
; PURPOSE:
;    reads, cleans and reformats images to a 'cactusmap' and a 256x256 image and changes the header into a cactusheader
; CATEGORY:
;    CACTUS
; INPUTS:
;    instrument 
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
; OUTPUTS:
;   '*_cactus.xdr.gz' files are saved in dir.in, every file is a stucture containing 2 images and the cactusheader
; COMMON BLOCKS: 
;   none
; PROCEDURE:
;
; CALLS:
;   cactus_image_select (for c2 and c3) and cor2_image_select for COR2 images
;   cactus_*_clean
;   cactus_polarmap
;   savestruct  ;ssw routine
; MODIFICATION HISTORY:
;  2008/04/03, ER, Created
;  2008/04/08, ER, insert cor2_image_select routine
;  2008/04/14, ER, preprocess also bad files and move them into dir.archive 
;  2008/05/16, JMK, fixed output filenames
;  2008/05/16, JMK, changed call to cactus_lasco_image_select
;  2008/05/16, ER, keyword no_logfile removed from call cactus_cor2_clean
;  2008/05/16, ER, take out 2nd image_select loop
;  2008/07/15, JMK, Removed median filtering
;  2008/08/29, JMK, Median filtering made optional with no_spatial_clean
;  2009/02/15, ER, Pass keyword spatial_clean instead of no_spatial_clean (more logical)
;  2009/11/18, ER, Only continue if nmyfiles gt 2 to avoid problems later on
;======================== end header ====================================================
PRO cactus_preprocess , no_logfile=no_logfile , instrument=instrument,spatial_clean=spatial_clean

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open
; ------------------------main code ------------------------------------
   IF NOT KEYWORD_SET(spatial_clean) THEN spatial_clean=0 ; set default to no median cleaning in space (it is slow)

; loop over detectors 
 detectors=cactus_detectors(instrument)
 for d=0,n_elements(detectors)-1 do begin
   printf, log, '  Now working on detector ', detectors[d]
   IF NOT FILE_TEST(dir.work+detectors[d]+'_image_list.sav') THEN $
    	  	printf, log, '  => No Input list for this detector.' $ 
   ELSE BEGIN
     restore, dir.work+detectors[d]+'_image_list.sav'   
     
     nmyfiles=n_elements(image_list)
     printf, log, '  Number of new files: ',strtrim(nmyfiles,2)   

     IF nmyfiles GT 2 THEN BEGIN
        ; select the 'good' myfiles
       CASE instrument of 
	  'SECCHI' : myfiles=cor2_image_select(image_list, badlist=badlist,nmyfiles) 
          'LASCO'  : myfiles=cactus_lasco_image_select(image_list, detectors[d],no_logfile=no_logfile,nmyfiles)
       ENDCASE
        nbad=n_elements(badlist)
    	if nbad gt 0 then begin
        	printf, log, '  Files that were refused:'
        	for i=0,nbad-1 do printf, log, '   ',badlist[i]
        endif
        printf, log, '  Number of good new files: ',strtrim(nmyfiles,2)
       
     ENDIF ;nmyfiles gt 0

     ; only continue if there are GOOD new files
     IF nmyfiles GT 2 THEN BEGIN
        ; read and clean the myfiles  
       FOR i=0, nmyfiles-1 DO BEGIN
           printf, log, '  Processing file: ',myfiles[i],' at ',systime(/UTC), ' UT'
           CASE detectors[d] OF 
            'c2'   : cactus_c2_clean,  filename=myfiles[i], image,index,ncosmics,no_logfile=no_logfile,spatial_clean=spatial_clean
            'c3'   : cactus_c3_clean,  filename=myfiles[i], image,index,ncosmics,no_logfile=no_logfile,spatial_clean=spatial_clean
            'cor2' : cactus_cor2_clean,filename=myfiles[i], image,index,ncosmics,spatial_clean=spatial_clean 
           ENDCASE
           printf, log, '  ncosmics corrected=',ncosmics
           polarimage=cactus_polarmap(image, index, no_logfile=no_logfile)
           image=rebin(image, !cactusfov.npixelx,!cactusfov.npixely)
           file=strsplit(index.filename,'.',/extract)  &   filename=file[0]+'_cactus.xdr'
           savestruct, dir.in+detectors[d]+path_sep()+filename, {polarim:polarimage, im:image, hdr:index}, /gz
       ENDFOR ;nmyfiles

     ENDIF ; nmyfiles gt 0
 ENDELSE; file_test

endfor ;detectors
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END

