;======================== header ===============================================
;+
; NAME:
;   cactus_reldev
; PURPOSE:
;   Does all preprocessing that needs to be done on many images simultaneously.
;   - correct for exposure time fluctuations (cactus_remetf.pro)
;   - transfer to relative deviations (difference scaled by image)
;
; CATEGORY:
;   cactus
;
; INPUTS:
;   
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   cactus_remetf 
;   
; MODIFICATION HISTORY:
;   2002/03/12, ver 1.00, David Berghmans
;               created on the basis of  ...
;   2002/05/13, DB  for c3:    s      =replicate(1b,5,5)
;   2002/05/29, DB, correct for exposure time fluctuations (remetf.pro)
;   2002/05/30, DB, for c2:    s      =replicate(1b,5,5)
;   2002/08/23, DB, keyword /nosavefiles
;   2002/11/21, DB, introduced loop over instruments
;   2003/03/23, DB, now uses differences instead of background subtraction
;   2003/04/07, DB, bug in missing block treatment corrected
;   2003/04/24, DB, added merging of c2 and c3 data into a 'datacube'
;   2003/06/10, DB, pipe dir removed
;   2003/06/10, DB, reading of flags removed: assuming new images in any case
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/03/12, ER, remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/18, ER, extract the code for making the running difference in a separate program: rundiff 
;   2008/03/18, ER, extract the code for removing dark pixels in a separate program: cactus_dark_blocks
;   2008/03/18, ER, Retrieve detectors (c2/c3/COR2) form function cactus_detectors
;   2008/03/19, ER, Quit IDL if input files do not exist
;   2008/03/20, ER, Merge detectors of same instrument in separate routine: cactus_merge_detectors
;   2008/05/22, ER, Delete detectors[d]+retide.sav at the end of the cactus_reldev, it is not used further
;   2009/02/15, ER, remove *_ar_images.sav, is not used further
;   2009/02/17, ER, correcting typo in allow_nonexistEnt
;   2009/04/14, ER, move retide=2.5*(retide<0.02) from rundiff_cube to here
;   2009/04/14, ER, take out all operations on xycube, these are only for viewing ==> done in cactus_diffmovie.pro
;   2009/11/09, ER, replace exit statement with GOTO, jump to avoid jumping outside IDL
;
;-
;======================== end header ===========================================
PRO cactus_reldev, no_logfile=no_logfile, instrument=instrument

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------main code ------------------------------------

 ; loop over detectors 
      detectors=cactus_detectors(instrument)
   for d=0,n_elements(detectors)-1 do begin
     printf, log, '  Now working on detector ', detectors[d]
     ; read input
     IF NOT FILE_TEST(dir.work+detectors[d]+'index.sav') THEN BEGIN
       printf, log, "  Input files do not exist. Jumping to end of program."
       GOTO, jump 
     ENDIF

     
     restore, dir.work+detectors[d]+'index.sav'  	; yields index.obstime
     restore, dir.work+detectors[d]+'_ar_images.sav' 	; yields polar

     if instrument eq 'LASCO' then begin
   	; now Remove Exposure Time Fluctuations from polar and xycube 
        cactus_remetf, index, polar, no_logfile=no_logfile
   	; remove too dark pixels from polar
     	cactus_dark_blocks, polar,no_logfile=no_logfile
     endif 

   ; make RElative TIme DErivative of intensity (ie (1/I)*dI/dt) : retide
     retide=rundiff_cube(index,polar,/ratio)
     retide=2.5*(retide<0.02)   ; to get same range as before with reldev (taken out of rundiff_cube)

     save, filename=dir.work+detectors[d]+'retide.sav', retide          ;new name: _ar_rundiff.sav (polar)?
   endfor   	; detectors
 ; merge instruments of same detector together
   cactus_merge_detectors, instrument=instrument, retide, index, no_logfile=no_logfile
   ;for d=0,n_elements(detectors)-1 do file_delete, dir.work+detectors[d]+'retide.sav' ; is created in this program, may be deleted after merging

 ; sort in time
   sz=size(retide)
   npixel1=sz[1]
   npixel2=sz[2]
   nimages=sz[3]
 
   good_order=sort(index.obstime)
   index=index[good_order]
   datacube=FLTARR(npixel1,npixel2,nimages)
   for i=0,nimages-1 do datacube[*,*,i]=retide[*,*,good_order[i]]

 ; save 
   save, filename=dir.work+'datacube.sav'	, datacube
   save, filename=dir.work+'index.sav'	, index

 ; remove *_ar_images.sav, is not used further, it makes free ~250M for 1 month of cor2 data
   ;for d=0,n_elements(detectors)-1 do file_delete,dir.work+detectors[d]+'_ar_images.sav',/allow_nonexistent
   printf, log, 'Files deleted: '+dir.work+'*_ar_images.sav'

   jump:

; ------------------------closing remarks-------------------------------
  cactus_loginfo, myname, log, no_logfile=no_logfile, /close

END
