;============================= header ===============================================
;+
; NAME:
;   cactus_velmap
; PURPOSE:
;   find ridges in r, \theta, t maps
; CATEGORY:
;   cactus 
; INPUTS:
; 
; KEYWORD PARAMETERS:
;   /no_logfile: to print diagnostics to a no_logfile (default=stdout=screen)
;   set thresh=minpeak to define the intensity threshold, default is 0.3
; OUTPUTS:
;
; COMMON BLOCKS: 
;   none
;
; CALLS:
;   calls the_total.so (in C)
;
; MODIFICATION HISTORY:
;   2002/05/12, ver 1.00, David Berghmans
;               created on the basis of  spwp/find_cme
;   2002/05/23, DB, now writing flags for catalo
;   2002/05/31, DB, merged with find_ridges
;   2003/04/02, DB, the procedure that produces the vectors has changed
;   2003/04/08, DB, maxniterations= 100, minpeak = 0.5
;   2003/04/25, DB, now working on merged c2 c3 slices
;   2003/06/10, DB, pipe dir removed    
;   2003/06/13, DB, can work now without flags, minvel, maxvel added
;   2003/06/26, DB, minpeak=0.3
;   2004/03/30, ER, calculate cadence using intervals ov 10 c2&c3 images combined
;   2004/03/30, ER, do not accept vectors of maxDT and minDT (to avoid wrong signal creating a CME)
;   2004/08/31, BN, speed up vector-part+while-loop by implementing a C script
;     	    	    dti=1/dti,remove abs-function,change vi>v in inner loop, make search-interval for ti [t1:t2]
;     	    	    this divides the velmap runtime with about a factor 6
;   2004/09/02, ER, re-define minDT and maxDT locally (i.e. for each t0 separately) s.t. we only allow the 
;   	    	    speeds in the requested range (vector-part)
;   2007/12/10, ER, introduce !cactus for defining directories, all programs now in same directory
;   2007/12/14, ER, myname from scope_traceback and cactus_loginfo routine introduced 
;   2007/12/19, ER, make intensity threshold a keyword. Default is original value of minpeak.
;   2008/01/03, ER, cactus_preamble introduced
;   2008/01/09, ER, maindir=maindir added
;   2008/02/07, ER&BN, introduce cactus_total.dlm
;   2008/03/12, ER, remove crontab keyword
;   2008/03/12, ER, maindir keyword removed, only needed in cactus_settings now
;   2008/03/14, ER, extract 'absolute' cactus parameters from !cactusfov and not from fov.sav 
;   2008/03/19, ER, Quit IDL if input files do not exist
;   2008/03/20, ER, Add maxniterations as keyword
;   2008/03/31, ER, bug repaired, cactus_total added before the WHILE loop
;   2008/05/15, ER, make threshold dependent on velmap-statistics, i.e. detect signal gt mean+factor*sigma; factor is a keyword, default = 3
;   2008/05/22, ER, Do not save cleanslices.sav anymore, it is not used further
;   2008/05/22, ER, restore datacube instead of slices (not saved anymore to save disk space), add temporary to save memory
;   2009/10/29, ER, initialize peak with minpeak+1 (instead of peak=1; minpeak may be larger than 1 for bright CMEs and simulations)
;   2009/11/09, ER, replace exit statement with GOTO, jump to avoid jumping outside IDL
;-
;======================== end header ===========================================
PRO cactus_velmap, maxniterations=maxniterations, no_logfile=no_logfile, factor=factor, threshold=threshold

   cactus_preamble, dir,myname
   cactus_loginfo, myname, log, no_logfile=no_logfile, /open

; ------------------------ keyword settings ----------------------------
   IF NOT KEYWORD_SET(threshold)	THEN threshold = 0.3
   IF NOT KEYWORD_SET(factor)           THEN factor = 3
   IF NOT KEYWORD_SET(maxniterations) 	THEN maxniterations=300
; ------------------------main code ------------------------------------
 
     ; read data
   IF NOT FILE_TEST(dir.work+'datacube.sav') OR NOT FILE_TEST(dir.work+'index.sav') THEN BEGIN
     printf, log, "  Input files do not exist. Jumping to end of program."
     GOTO, jump
   ENDIF

	  restore, dir.work + 'datacube.sav'  ; yields datacube
          slices=transpose(temporary(datacube), [2,1,0])
	  restore, dir.work + 'index.sav'   ; yields index.obstime
	  nimages  = (size(slices))[1]
	  nradii   = (size(slices))[2]   
	  nangles  = (size(slices))[3]   
	  slices=slices<(0.05)  ; this weakens the influence of individual very bright points (planets, comets)
 
         ; determine which velocities should be considered
	  minvel =  100 ; (km/s) the minimal projected speed considered for a CME
	  maxvel = 2100 ; (km/s) the maximal projected speed considered for a CME
          max_transit_time = (nradii*!cactusfov.km_per_pix) / minvel ; in sec
          min_transit_time = (nradii*!cactusfov.km_per_pix) / maxvel 
	                   
    	  v_min=minvel*(24*3600./!cactusfov.km_per_pix)  ; in pixels/obstime
    	  v_max=maxvel*(24*3600./!cactusfov.km_per_pix)
	  t_max=nradii/v_min	    	    	    	; in obstime
	  t_min=nradii/v_max

	 ; typical number of seconds between two subsequent images
	  cadence = 24.*3600.*median(index[10:*].obstime-index[0:(nimages-10)].obstime)/10
	  	    	
	  maxDT = FIX(max_transit_time/cadence)
	  minDT = FIX(min_transit_time/cadence)

    	; calculate true velocities
    	  minvel=(nradii*!cactusfov.km_per_pix)/(maxDT*cadence)
    	  maxvel=(nradii*!cactusfov.km_per_pix)/(minDT*cadence)

	  printf, log, '   minvel   (km/s)= ',minvel
	  printf, log, '   maxvel   (km/s)= ',maxvel
	  printf, log, '   cadence   (min)= ',FIX(cadence/60.)
	  printf, log, '   minDT (#images)= ',minDT
	  printf, log, '   maxDT (#images)= ',maxDT
	  printf, log, '   maxniterations = ',maxniterations
     
        ; save velocity detection settings
	  save, filename=dir.work+'velocity_settings.sav', $
	          minvel, maxvel, cadence, minDT, maxDT 
   
 	; DT (minDT -> maxDT) is the time it takes to move the full radial range DR with a 
	; speed v
	  DR=float(nradii)   ; moet 'nradii-1' zijn
	  nDT   = maxDT-minDT+1
	  ntimes =  nimages + maxDT

        ; we must fake extra data 
	  dt = index[nimages-1].obstime - index[nimages-2].obstime
	  newindex=index[nimages-1]
	  FOR i = 0, maxDT-1 DO BEGIN
	      newindex.obstime=newindex.obstime+dt
	      index=[index, newindex]
          ENDFOR

	; make vectors
	  printf, log, '   Creating vectors '
	  vectors=ULONARR(nradii,nDT,nimages)
	  for t0=0, nimages-1 do begin
        	dti = (index.obstime-index[t0].obstime) 
		dti[t0]=-1  ; otherwise we divide by zero
		;calculate DT-range locally
   	  	t_first= min(where(dti ge t_min))
       	    	t_last = max(where(dti le t_max))
	    	local_minDT = max([t_first-t0+1,minDT])
	    	local_maxDT = min([t_last-t0+1,maxDT])
	    	;printf,log,'   for t0 = ',strtrim(t0,2), ': DT-range allowed is [',strtrim(local_minDT,2),':',strtrim(local_maxDT,2),']' 
   	    	dti = 1.0/dti     	   
        	for DT=local_minDT,local_maxDT do begin 
		  v= nradii/(index[t0].obstime-index[t0+DT].obstime)
      		  ; ri=0
		  vectors[0,DT-minDT,t0]=t0
		  ; other ri, first define search-interval
      	    	  mindiff = min(abs(dti + v), t1)
      	    	  vectors[1,DT-minDT,t0]= ntimes+t1
      	    	  mindiff = min(abs((nradii - 1) * dti + v), t2)
      	    	  vectors[nradii - 1,DT-minDT,t0]= (nradii - 1)*ntimes+t2
      	    	  ddti = dti[t1:t2]
      	    	  v=v+ddti   ; v now is converted into a vector instead of a double
		  for ri=2, nradii-2 do begin
                    ; find appropiat ti
		     v=v+ddti
                     mindiff =  min(v*v,ti)
      		     vectors[ri,DT-minDT,t0]=  ri*ntimes+ti+t1
		  endfor   
		endfor
	  endfor
       ; write vectors
	 save, filename=dir.work+'vectors.sav', vectors, maxDT, minDT

       ; declarations & initialisations
	 velmap      = FLTARR(nimages,  nDT)
	 tvanglecube = FLTARR(nimages,nDT,nangles) 
       ; loop over angles
	 printf, log, '   Entering loop over angles'
	 for angle= 0, nangles-1 do begin
            tvangle     = FLTARR(nimages,  nDT)
            myxt= FLTARR(nimages+maxDT,nradii) 
            myxt[0:(nimages-1),*]=slices[*,*,angle]  
          ; limited to  weaken the influence of individual very bright points (planets, comets)
	  ; initialise velmap (equivalent of accumulator space of Hough transform)  
          ; uses C-script
            cactus_total, velmap, myxt, vectors, 0,nimages-1
	  ; take out most significant peak and iterate   
            dummy=0
	    r=moment(velmap(where(velmap gt 0)),sdev=stdev)
	    minpeak =max([r[0]+factor*stdev,threshold]) & peak=minpeak+1
            WHILE dummy LT maxniterations and peak gt minpeak DO BEGIN
               peak     = max(velmap[0:nimages-1,*],peakpos) 
               tvangle[peakpos]=peak    
               ypeakpos = peakpos/nimages
               xpeakpos = peakpos - ypeakpos*nimages
               myxt[vectors[*,ypeakpos,xpeakpos]]=0.0
    	       ;remove vector from data and find peak in velmap (uses C-script)
               cactus_total, velmap, myxt, vectors, xpeakpos+ypeakpos-maxDT,xpeakpos+ypeakpos
               dummy=dummy+1
	       r=moment(velmap(where(velmap gt 0)),sdev=stdev)
	       minpeak =max([r[0]+factor*stdev,threshold])
            ENDWHILE
	    ;printf, log, angle,' :n_iterations is ',strtrim(dummy,2) 
	    tvanglecube[*,*,angle]=tvangle
	  endfor  ; angles

    	; exclude the extreme velocities, minDT'=minDT+1 and maxDT'=maxDT-10
    	   tvanglecube[*,0,*]=0.0 & tvanglecube[*,nDT-10:nDT-1,*]=0.0
	; save output
	  save, filename=dir.work+'tvanglecube.sav', tvanglecube

    jump:
 ; ------------------------closing remarks-------------------------------
 cactus_loginfo, myname, log, no_logfile=no_logfile, /close


END
