;+
; NAME: makemov.pro
;
; PURPOSE:
;   make a movie of a series of images  
;  
; SAMPLE CALLING SEQUENCE:
;   result=makemov(imgdir,outfile=outfile,ffmpegdir=ffmpegdir,bitrate=bitrate,overwrite=overwrite)
; INPUT:
;   imgdir = directory containing imgfiles
;   [outfile] = movie.ext filename to write movie to ; default imgdir+'../test.mov'
;   [ffmpegdir] = directory where ffmpeg is located if not in $PATH
;   [bitrate] = bitrate for movie ; default 500k
;   [overwrite] = set to 1 to overwrite already existing movie.
;
; OUTPUT:
;    result = 1 if succesful, -1 if failed
;    MOVIE [outfile]
; 
; CALLS:
;   strnr.pro routine
;   /tmp/ directory on system
;   ffmpeg binary
;
; HISTORY:
;   2008/01/15, JMK, Created based no makeMov script
;   2008/01/16, JMK, Added Libary info
;   2008/01/22,  ER, compare time of movie with max time of images, they are not necessarily made chronologically
;   2008/04/01,  BN, pass ffmpegoptions before the input specifier because this is used by calling routines to set 
;                    the frame rate (should actually be a separate argument)
;   2008/05/22,  ER, replace spawn commands with IDL equivalents
;   2008/05/22,  ER, add keyword tempdir, usd dir.tmp when calling makemov inside cactus
;   2008/06/11  JMK, removed ffmpegdir part of spawn (as ffmpeg should now be in path)
;-
;==============================================================
FUNCTION makemov,imgdir,tempdir=tempdir,outfile=outfile,ffmpegdir=ffmpegdir,bitrate=bitrate,$
	overwrite=overwrite,infiles=infiles,ffmpegoptions=ffmpegoptions

on_error,2

if not keyword_set(bitrate) then bitrate='512k'
if not keyword_set(tempdir) then tempdir='/tmp/makemov/'
if not keyword_set(ffmpegdir) then ffmpegdir=''  
if not keyword_set(outfile) then outfile=imgdir+'/../test.mov'
if not keyword_set(infiles) then infiles='/*'
if not keyword_set(ffmpegoptions) then ffmpegoptions=" "

files=file_search(imgdir+infiles,count=count)     
if count eq 0 then begin
	print,'No image files found in '+imgdir+infiles
	return,-1
endif

imgformat=strmid(files[0],strpos(files[0],'.',/reverse_search))

if file_test(outfile) and not keyword_set(overwrite) then begin
    mtime_movie=(file_info(outfile)).mtime        ; check time movie was created
    mtime_files=(file_info(files)).mtime ; check time last image was created
    if max(mtime_files) lt mtime_movie then begin      
        print,'File already exists and uptodate: ',outfile
        print,'Quitting'
        return,-1
    endif
endif

print,'Creating movie from '+imgdir
print,'Consisting of '+strtrim(count,2)+' '+imgformat+' images'
print,'Output goes to '+outfile
file_mkdir, tempdir   ; Create temp dir
for ifile=0,count-1 do file_copy, files[ifile],tempdir+string(ifile, format='(I06)')+imgformat   ; cp files for right format
print,'-----'

spawn,'ffmpeg -y '+ffmpegoptions+' -i '+tempdir+'%06d'+imgformat+$
    	  ' -b '+bitrate+' '+outfile   ; do the ffmpeg (-y for overwrite)
	  
file_delete, tempdir, /recursive

if file_test(outfile) then return,1 else return,-1    
end    
